%module fftw
%{
#include "fftw.h"      /* This one has fftw_real = double */
#include "rfftw.h"
#include "Python.h"
#include "arrayobject.h"

void fftw_wrap(fftw_plan plan, int howmany, 
               fftw_complex *in, int ioffset, int istride, int idist,
               fftw_complex *out, int ooffset, int ostride, int odist)
{
    fftw(plan, howmany, 
         in+ioffset, istride, idist, 
         out+ooffset, ostride, odist);
};

void fftwnd_wrap(fftwnd_plan plan, int howmany,
		   fftw_complex *in, int ioffset, int istride, int idist,
		   fftw_complex *out, int ooffset, int ostride, int odist)
{
    fftwnd(plan, howmany, 
           in+ioffset, istride, idist,
           out+ooffset, ostride, odist);
}

void fftw_ip_wrap(fftw_plan plan, int howmany, 
               fftw_complex *in, int ioffset, int istride, int idist)
{
    fftw(plan, howmany, 
         in+ioffset, istride, idist, 
         (fftw_complex*)NULL, 0, 0);
}

void fftwnd_ip_wrap(fftwnd_plan plan, int howmany,
		   fftw_complex *in, int ioffset, int istride, int idist)
{
    fftwnd(plan, howmany, 
           in+ioffset, istride, idist,
           (fftw_complex*)NULL, 0, 0);
}

void rfftwnd_real_to_complex_wrap
( rfftwnd_plan plan, int howmany,
  fftw_real *in, int ioffset, int istride, int idist,
  fftw_complex *out, int ooffset, int ostride, int odist )
{
    rfftwnd_real_to_complex(plan, howmany, 
			    in+ioffset, istride, idist,
			    out+ooffset, ostride, odist);
}

void rfftwnd_complex_to_real_wrap
( rfftwnd_plan plan, int howmany,
  fftw_complex *in, int ioffset, int istride, int idist,
  fftw_real *out, int ooffset, int ostride, int odist )
{
    rfftwnd_complex_to_real(plan, howmany,
			    in+ioffset, istride, idist,
			    out+ooffset, ostride, odist);
}

void rfftwnd_real_to_complex_ip_wrap
( rfftwnd_plan plan, int howmany, 
  fftw_real *in, int ioffset, int istride, int idist ) 
{
    rfftwnd_real_to_complex(plan, howmany,
			    in+ioffset, istride, idist,
			    (fftw_complex *)NULL, 0, 0);
}

void rfftwnd_complex_to_real_ip_wrap
( rfftwnd_plan plan, int howmany, 
  fftw_complex *in, int ioffset, int istride, int idist ) 
{
    rfftwnd_complex_to_real(plan, howmany,
			    in+ioffset, istride, idist,
			    (fftw_real *)NULL, 0, 0);
}
%}

%init %{
    import_array();
%}

%typemap(python,in) fftw_complex * {
    PyArrayObject *arr;
    arr = (PyArrayObject *) $source;
    if ( !PyArray_Check($source) 
	 || !PyArray_ISCONTIGUOUS(arr)
	 || arr->descr->type_num != PyArray_CDOUBLE ) {
	PyErr_SetString(PyExc_TypeError,"Argument must be a contiguous Double Complex NumPy array.");
	return NULL;
    }
    $target = (fftw_complex *)arr->data;
}

%typemap(python,in) fftw_real * {
    PyArrayObject *arr;
    arr = (PyArrayObject *) $source;
    if ( !PyArray_Check($source)
	 || !PyArray_ISCONTIGUOUS(arr)
	 || arr->descr->type_num != PyArray_DOUBLE ) {
        PyErr_SetString(PyExc_TypeError,"Argument must be a contiguous Double NumPy array.");
        return NULL;
    }
    $target = (fftw_real *)arr->data;
}

%typemap(python,in) int * {
    PyArrayObject *arr;
    arr = (PyArrayObject *) $source;
    if ( !PyArray_Check($source)
	 || !PyArray_ISCONTIGUOUS(arr)
	 || arr->descr->type_num != PyArray_INT ) {
        PyErr_SetString(PyExc_TypeError,"Argument must be an integer NumPy array ('l' or 'i').");
        return NULL;
    }
    $target = (int *)arr->data;
}

%typemap(python,in) FILE * {
    if (!(PyFile_Check($source))) {
        PyErr_SetString(PyExc_TypeError, "Argument must be a File");
        return NULL;
    }
    $target = PyFile_AsFile($source);
}

typedef enum {
    FFTW_FORWARD = -1, FFTW_BACKWARD = 1
} fftw_direction;

extern const char *fftw_version;

/*****************************
 *        Plans
 *****************************/
/*
 * A plan is a sequence of reductions to compute a FFT of
 * a given size.  At each step, the FFT algorithm can:
 *
 * 1) apply a notw codelet, or
 * 2) recurse and apply a twiddle codelet, or
 * 3) apply the generic codelet.
 */
/* flags for the planner */
#define FFTW_ESTIMATE (0)
#define FFTW_MEASURE  (1)
#define FFTW_OUT_OF_PLACE (0)
#define FFTW_IN_PLACE (8)
#define FFTW_USE_WISDOM (16)
#define FFTW_THREADSAFE (128)  /* guarantee plan is read-only so that the
				  same plan can be used in parallel by
				  multiple threads */
#define FFTWND_FORCE_BUFFERED (256)	/* internal, undocumented flag */

/****** fftw ******/
extern fftw_plan fftw_create_plan_specific(int n, fftw_direction dir,
					   int flags,
					   fftw_complex *in, int istride,
					 fftw_complex *out, int ostride);
extern fftw_plan fftw_create_plan(int n, fftw_direction dir, int flags);
extern void fftw_print_plan(fftw_plan plan);
extern void fftw_fprint_plan(FILE *f, fftw_plan plan);
extern void fftw_destroy_plan(fftw_plan plan);

extern void fftw(fftw_plan plan, int howmany, fftw_complex *in, int istride,
		 int idist, fftw_complex *out, int ostride, int odist);

/****** fftwnd ******/
typedef fftwnd_data *fftwnd_plan;
extern fftwnd_plan fftwnd_create_plan(int rank, const int *n,
				      fftw_direction dir,
				      int flags);
extern fftwnd_plan fftwnd_create_plan_specific(int rank, const int *n,
					       fftw_direction dir,
					       int flags,
					   fftw_complex *in, int istride,
					 fftw_complex *out, int ostride);
/* Freeing the FFTWND plan: */
extern void fftwnd_destroy_plan(fftwnd_plan plan);
/* Printing the plan: */
extern void fftwnd_fprint_plan(FILE *f, fftwnd_plan p);
extern void fftwnd_print_plan(fftwnd_plan p);
/* Computing the N-Dimensional FFT */
extern void fftwnd(fftwnd_plan plan, int howmany,
		   fftw_complex *in, int istride, int idist,
		   fftw_complex *out, int ostride, int odist);

/******  rfftw ******/
typedef fftw_plan rfftw_plan;
extern rfftw_plan rfftw_create_plan_specific(int n, fftw_direction dir,
					    int flags,
					    fftw_real *in, int istride,
					    fftw_real *out, int ostride);
extern rfftw_plan rfftw_create_plan(int n, fftw_direction dir, int flags);
extern void rfftw_destroy_plan(rfftw_plan plan);
extern void rfftw_fprint_plan(FILE *f, rfftw_plan p);
extern void rfftw_print_plan(rfftw_plan p);

extern void rfftw(rfftw_plan plan, int howmany, fftw_real *in, int istride,
		  int idist, fftw_real *out, int ostride, int odist);

/****** rfftwnd ******/
typedef fftwnd_plan rfftwnd_plan;
#define FFTW_REAL_TO_COMPLEX FFTW_FORWARD
#define FFTW_COMPLEX_TO_REAL FFTW_BACKWARD
extern rfftwnd_plan rfftwnd_create_plan_specific(int rank, const int *n,
						fftw_direction dir, int flags,
						fftw_real *in, int istride,
						fftw_real *out, int ostride);
extern rfftwnd_plan rfftwnd_create_plan(int rank, const int *n,
					fftw_direction dir, int flags);
extern void rfftwnd_destroy_plan(rfftwnd_plan plan);
extern void rfftwnd_fprint_plan(FILE *f, rfftwnd_plan plan);
extern void rfftwnd_print_plan(rfftwnd_plan plan);

extern void rfftwnd_real_to_complex(rfftwnd_plan p, int howmany,
				    fftw_real *in, int istride, int idist,
				    fftw_complex *out, int ostride, int odist);
extern void rfftwnd_complex_to_real(rfftwnd_plan p, int howmany,
				    fftw_complex *in, int istride, int idist,
				    fftw_real *out, int ostride, int odist);


/***********Some Common Codes for FFTW, RFFTW************/
extern void fftw_check_memory_leaks(void);
extern void fftw_print_max_memory_usage(void);
/* Wisdom: */
/*
 * define this symbol so that users know we are using a version of FFTW
 * with wisdom
 */
extern void fftw_forget_wisdom(void);
extern void fftw_export_wisdom_to_file(FILE *output_file);
extern fftw_status fftw_import_wisdom_from_file(FILE *input_file);
extern char *fftw_export_wisdom_to_string(void);
extern fftw_status fftw_import_wisdom_from_string(const char *input_string);

/****** wrappers added by M Xu ******/
void fftw_wrap(fftw_plan plan, int howmany, 
               fftw_complex *in, int ioffset, int istride, int idist,
               fftw_complex *out, int ooffset, int ostride, int odist);
void fftwnd_wrap(fftwnd_plan plan, int howmany,
		 fftw_complex *in, int ioffset, int istride, int idist,
		 fftw_complex *out, int ooffset, int ostride, int odist);
void fftw_ip_wrap(fftw_plan plan, int howmany, 
		  fftw_complex *in, int ioffset, int istride, int idist);
void fftwnd_ip_wrap(fftwnd_plan plan, int howmany,
		   fftw_complex *in, int ioffset, int istride, int idist);

void rfftwnd_real_to_complex_wrap
( rfftwnd_plan plan, int howmany,
  fftw_real *in, int ioffset, int istride, int idist,
  fftw_complex *out, int ooffset, int ostride, int odist );

void rfftwnd_complex_to_real_wrap
( rfftwnd_plan plan, int howmany,
  fftw_complex *in, int ioffset, int istride, int idist,
  fftw_real *out, int ooffset, int ostride, int odist );

void rfftwnd_real_to_complex_ip_wrap
( rfftwnd_plan plan, int howmany, 
  fftw_real *in, int ioffset, int istride, int idist );

void rfftwnd_complex_to_real_ip_wrap
( rfftwnd_plan plan, int howmany, 
  fftw_complex *in, int ioffset, int istride, int idist );







